<?php
/**
 * MetaData module.
 * This module contains the methods to calculate metadata from raw XHTML data.
 * @package Basics
 * @link http://www.estratecms.org/
 * @author Bas Jobsen <bas@emailcommunications.nl>
 * @license http://www.gnu.org/licenses/gpl.html
 * @copyright Copyright (c) 2005, Estrate, Rudolph Froger, The Netherlands
 */

/**
 * Automagically create metadata from XHTML text.
 * Provides two public methods:
 * 1. getKeywords which generates keywords from XHTML input.
 * 2. getSummary which generates a summary from XHTML input.
 * You can use these methods for the metadata tags in XHTML:
 * getKeywords can be used for the keywords metatag. They will be used by a small number of searchengines.
 * getSummary can be used for the description metatag. This is used a lot by searchengines.
 * You can also use this methods for other tasks (such as a searchfunction or a indexing service).
 */
class metadata
{
    /**
     * Sort words by length and return top length keywords.
     * @param array $input An array with all keywords.
     * @param integer $nr Number of keywords to return, defaults to 25.
     */
    function _sortStrLen($input, $nr = 25)
    {
        $new = array();
        foreach($input as $value)
        {
            $new[$value] = strlen($value);       
        }
        arsort($new,SORT_NUMERIC);
        reset($new);
        $keywords = '';
        for($i=0; $i<min($nr,count($new)); $i++)
        {
            $key = key($new);
            if (strlen($key)<4) {
                break;
            }
            if ($i>0) {
                $keywords .=', ';
            }
            $keywords .= $key;
            next($new);
        }
        return $keywords;
    }

    /**
     * Clean XHTML from tags, entities and hyperlinks.
     * @param string $xhtml XHTML input
     * @return string Returns text without any XHTML elements.
     */
    function _cleanXHTML($xhtml)
    {
        $xhtml = strip_tags($xhtml);
        $xhtml = str_replace(array("\n", "\r", '&nbsp;'), ' ', $xhtml);
        $xhtml = html_entity_decode($xhtml, ENT_QUOTES);
        //replace decimal and heximal entities because html_entity_decode seems to ignore them
        $xhtml = preg_replace('~&#x([0-9a-f]+);~ei', 'chr(hexdec("\\1"))', $xhtml);
        $xhtml = preg_replace('~&#([0-9]+);~e', 'chr(\\1)', $xhtml);
        //replace multiple spaces by one space
        $xhtml = preg_replace('/(\s+)/', ' ', $xhtml);
        //remove webaddress
        $xhtml = preg_replace('/(www\.[^ ]+)/i', '', $xhtml);
        $xhtml = preg_replace('/(http:\/\/([^ ^<]+))/i', '', $xhtml);
        //remove e-maildresses
        $xhtml = preg_replace('/([a-z0-9\._-]+@[a-z0-9\.-]+[a-z]{2,4})/i', '', $xhtml);
        return $xhtml;
    }   

    /**
     * Automagically generate keywords from XHTML text.
     * @param string $xhtml XHTML input.
     * @param integer $nr Number of keywords te return, defaults to 25.
     * @return array Returns an array of keywords
     */
    function getKeywords($xhtml, $nr = 25)
    {
        $xhtml = $this->_cleanXHTML($xhtml);
        $xhtml = preg_replace('/[.,?!:;()" ]+/', ' ', $xhtml);
        $xhtml = htmlentities($xhtml, ENT_QUOTES);
        $keywords = explode(' ', $xhtml);
        return $this->_sortStrLen($keywords, $nr);
    }   

    /**
     * Automagically create a summary from a XHTML text.
     * It extracts the first most important sentences from the source.
     * @param string $xhtml XHTML input.
     * @param integer $maxlength Maximal string length of the summary.
     * @return string Returns the summary from the provided XHTML input.
     */
    function getSummary($xhtml, $maxlength = 256, $_fix = FALSE)
    {
        //convert lists into sentences
        $xhtml = preg_replace('/(.*)(<\/li>)([^<]+)(<\/[ou]l>)(.*)/m','$1. $5', $xhtml);
        $xhtml = preg_replace('/(.*)(<\/li>)(.*)/','$1, $3', $xhtml);
        $xhtml = preg_replace('/(.*)(<li>|<[ou]l>)(.*)/','$1$3', $xhtml);
        //remove hyperlinks
        $xhtml = preg_replace('/(.*)(<a [^>]+>)([^<>]+)(<\/a>)(.*)/','$1 $3 $5', $xhtml);
        $xhtml = $this->_cleanXHTML($xhtml);
        //Split into sentences
        $sentences_points = preg_split('/([.?!:])+/', $xhtml, -1, PREG_SPLIT_DELIM_CAPTURE);
        
        for ($i=0; $i < count($sentences_points); $i+=2) {
            $sentences[] = trim($sentences_points[$i].(isset($sentences_points[$i+1]) ? $sentences_points[$i+1] : ''));
        }
        
        $summary = '';
        $k = 0;
        for ($i=0; $i < count($sentences); $i++) {
            if (strlen($sentences[$i]) > 25) {
                if ((strlen($summary)+strlen($sentences[$i])+1) < $maxlength) {
                    if ($k>0) {
                        $summary.=' ';
                    }
                    $summary .= $sentences[$i];
                    $k++;
                } elseif ($k>0) {
                    break;
                }
            }
        }
        if (empty($summary) && !$_fix) {
            //getting a summary failed; try to get sentences splitted by comma
            return $this->getSummary(str_replace(',','.',$xhtml), $maxlength, TRUE);
        } else {
            return htmlentities($summary, ENT_QUOTES);
        }
    }   
}
?>
